package com.css.simulation.oauth.client.controller;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.SignSingleParameter;
import api.common.pojo.param.SignUsernameParameter;
import api.common.pojo.po.UserPO;
import api.common.pojo.vo.SimulationTokenVO;
import api.common.util.*;
import com.css.simulation.oauth.client.configuration.oauth.OauthParameter;
import com.css.simulation.oauth.client.mapper.UserMapper;
import com.css.simulation.oauth.client.util.EncodeUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.util.Set;

@Controller
@RequestMapping("/sign")
@Slf4j
public class SignController {

    static final String USER_ID = "simulation-oauth-client";

    @Resource
    UserMapper userMapper;
    @Autowired
    CloseableHttpClient closeableHttpClient;
    @Autowired
    RequestConfig requestConfig;
    @Autowired
    OauthParameter oauthParameter;

    @Autowired
    StringRedisTemplate redisTemplate;

    @RequestMapping("/hello")
    @ResponseBody
    public String hello() {
        return "hello clent!";
    }

    @RequestMapping("/entry")
    @SneakyThrows
    public String entry(@RequestParam("ticket") String ticket) {
        log.info("------- /entry 接收到的 ticket 为：" + ticket);
        String encodeTicket = EncodeUtil.utf8(ticket);
        log.info("------- /entry ticket 编码之后为：" + encodeTicket);
        log.info("------- /entry 重定向地址为：" + oauthParameter.getZoogooyRedirectUri());
        String encodeRedirect = EncodeUtil.utf8(oauthParameter.getZoogooyRedirectUri());
        log.info("------- /entry 重定向地址编码之后为：" + encodeRedirect);

        String zoogooyAuthorizeUrl = oauthParameter.getZoogooyAuthorizeUri() +
                "?appid=" + oauthParameter.getZoogooyAppid() +
                "&redirect_uri=" + encodeRedirect +
                "&ticket=" + encodeTicket +
                "&response_type=code" +
                "&scope=snsapi_userinfo";
        log.info("------- /entry 需要访问的授权地址为：" + zoogooyAuthorizeUrl);
        return "redirect:" + zoogooyAuthorizeUrl;
    }

    /**
     * 单点登录
     * 众工业平台：http://icvdc.china-icv.cn/
     *
     * @return token 信息
     */
    @RequestMapping("/single")
    @SneakyThrows
    @ResponseBody
    public ResponseBodyVO<SimulationTokenVO> single(@RequestBody SignSingleParameter signSingleParameter) {

        String code = signSingleParameter.getCode();
        String encodeCode = EncodeUtil.utf8(code);
        String ticket = signSingleParameter.getTicket();
        String encodeTicket = EncodeUtil.utf8(ticket);
        log.info("------- /single 接收到的 code 为：" + code);
        log.info("------- /single code 编码之后为：" + encodeCode);
        log.info("------- /single 接收到的 ticket 为：" + ticket);
        log.info("------- /single ticket 编码之后为：" + encodeTicket);
        String zoogooyTokenUrl = oauthParameter.getZoogooyTokenUri() +
                "?appid=" + oauthParameter.getZoogooyAppid() +
                "&secret=" + oauthParameter.getZoogooyAppSecret() +
                "&code=" + encodeCode +
                "&grant_type=authorization_code";
        log.info("------- /single 根据编码之后的 code 获取众工业平台 access_token 的地址为：" + zoogooyTokenUrl);
        String zoogooyTokenJson = HttpUtil.get(closeableHttpClient, requestConfig, zoogooyTokenUrl);
        log.info("------- /single 根据编码之后的 code 获取众工业平台 access_token 的结果为：" + zoogooyTokenJson);
        ObjectMapper objectMapper1 = new ObjectMapper();
        JsonNode tokenRoot = objectMapper1.readTree(zoogooyTokenJson);
        if (!tokenRoot.path("success").asBoolean()) {
            throw new RuntimeException(tokenRoot.path("message").asText());
        }
        log.info("------- /single 众工业平台 token 信息为：" + zoogooyTokenJson);
        /*
        {
          "data" : {
            "scope" : "snsapi_userinfo",
            "expires_in_sec" : 60,
            "access_token" : "30b26a0824a1456f96bfc151b7bb5356",
            "openid" : "6c7a11cebMkd0qvhpskKX1nRzAicfH9xePJ945DOG+7j9KOB/9M=",
            "appid" : "2af6f44d98104dc5adcbfb49809ff9d5",
            "create_time" : "2022-03-25 17:50:49",
            "unionid" : "E2c47RsObMkd0qvhpskKX1nRzAicfH9xePJ945DOG+7j9KOB/9M="
          },
          "success" : true,
          "message" : "ok",
          "code" : 1,
          "nowTime" : "2022-03-25 17:50:52"
        }
         */

        String accessToken = tokenRoot.path("data").path("access_token").asText();
        String encodeAccessToken = EncodeUtil.utf8(accessToken);
        String openid = tokenRoot.path("data").path("openid").asText();
        String encodeOpenid = EncodeUtil.utf8(openid);

        log.info("------- /single 众工业平台 access_token 为：" + accessToken);
        log.info("------- /single 众工业平台 access_token 编码之后为：" + encodeAccessToken);
        log.info("------- /single 众工业平台 openid 为：" + openid);
        log.info("------- /single 众工业平台 openid 编码之后为：" + encodeOpenid);
        String zoogooyUserUrl = oauthParameter.getZoogooyUserUri() +
                "?access_token=" + encodeAccessToken +
                "&openid=" + encodeOpenid +
                "&ticket=" + encodeTicket;
        log.info("------- /single 根据编码之后的 access_token、openid、ticket 获取众工业平台用户信息：" + zoogooyUserUrl);
        /*
        {
          "data" : {
            "loginName" : "13389957835",
            "photoId" : "1648195226865/908fa0ec08fa513d7e3510cd40c346f1b3fbd9ba.jpeg",
            "userType" : "personal",
            "nickname" : "尼卡",
            "openid" : "6c7a11cebMkd0qvhpskKX1nRzAicfH9xePJ945DOG+7j9KOB/9M="
          },
          "success" : true,
          "message" : "ok",
          "code" : 1,
          "nowTime" : "2022-03-28 11:11:09"
        }
         */
        String zoogooyUserJson = HttpUtil.get(closeableHttpClient, requestConfig, zoogooyUserUrl);
        ObjectMapper objectMapper2 = new ObjectMapper();
        JsonNode userRoot = objectMapper2.readTree(zoogooyUserJson);
        if (!userRoot.path("success").asBoolean()) {
            throw new RuntimeException("------- /single 获取众工业平台用户信息出错：" + zoogooyUserJson);
        }
        log.info("------- /single 众工业平台用户信息为：" + zoogooyUserJson);

        String username = userRoot.path("data").path("loginName").asText();
        String nickname = userRoot.path("data").path("nickname").asText();
        String password = EncryptUtil.getLowerMD5(oauthParameter.getSimulationDefaultPassword());

        //3 使用 union_id 查询数据库，是否已在仿真平台存在该用户
        UserPO oldUser = userMapper.selectByOpenIdIgnoreDelete(openid);
        if (oldUser == null) {   //3-1 仿真平台不存在用户，直接创建新的
            UserPO newUser = new UserPO();
            newUser.setId(StringUtil.getRandomUUID());
            newUser.setUsername(username);
            newUser.setNickname(nickname);
            newUser.setOpenid(openid);
            newUser.setPassword(password);
            newUser.setCreateUserId(USER_ID);
            newUser.setModifyUserId(USER_ID);
            newUser.setCreateTime(TimeUtil.getNowForMysql());
            newUser.setModifyTime(TimeUtil.getNowForMysql());
            newUser.setIsDeleted("0");
            userMapper.insert(newUser);
            log.info("------- /single 成功插入新用户信息：" + newUser);
        } else {
            if ("1".equals(oldUser.getIsDeleted())) { //3-2 仿真平台存在删除状态用户，改为未删除
                userMapper.updateIsDeleted(openid, "0");
            }
            //3-3 仿真平台用户存在未删除用户，放行。不会每次都更新，防止在仿真云平台更新的信息被众工业覆盖
            username = oldUser.getUsername();
            password = oldUser.getPassword();
        }

        // -------------------------------- 这里需要用到前面创建的用户信息，所以不使用事务 --------------------------------
        //4 根据仿真平台用户名密码颁发仿真平台 token，返回给前端
        String simulationTokenUrl = oauthParameter.getSimulationTokenUri() +
                "?grant_type=password" +
                "&client_id=" + oauthParameter.getSimulationClientId() +
                "&client_secret=" + oauthParameter.getSimulationClientSecret() +
                "&username=" + username +
                "&password=" + password;
        log.info("------- /single 获取仿真云平台 token 信息：" + simulationTokenUrl);
        String simulationToken = HttpUtil.get(closeableHttpClient, requestConfig, simulationTokenUrl);
        log.info("------- /single 仿真云平台 token 信息为：" + simulationToken);
        SimulationTokenVO simulationTokenVO = JsonUtil.jsonToBean(simulationToken, SimulationTokenVO.class);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, simulationTokenVO);
    }


    /**
     * 用户名密码登录
     *
     * @return token 信息
     */
    @RequestMapping("/username")
    @SneakyThrows
    @ResponseBody
    public ResponseBodyVO<SimulationTokenVO> username(@RequestBody @Validated SignUsernameParameter signSingleParameter) {

        //1 使用 username 查询数据库，是否已在仿真平台存在该用户
        String username = signSingleParameter.getUsername();
        String password = signSingleParameter.getPassword();
        log.info("登录参数为：" + signSingleParameter);
        UserPO oldUser = userMapper.selectByUsername(username);
        if (oldUser == null) {   //2-1 仿真平台不存在用户，直接创建新的
            return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE, "不存在该用户!");
        } else { //2-2 仿真平台存在用户判断密码
            if (!oldUser.getPassword().equals(password)) {
                return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE, "密码错误！");
            }
        }

        //3 根据仿真平台用户名密码颁发仿真平台 token，返回给前端
        String simulationTokenUrl = oauthParameter.getSimulationTokenUri() +
                "?grant_type=password" +
                "&client_id=" + oauthParameter.getSimulationClientId() +
                "&client_secret=" + oauthParameter.getSimulationClientSecret() +
                "&username=" + username +
                "&password=" + password;
        String simulationToken = HttpUtil.get(closeableHttpClient, requestConfig, simulationTokenUrl);
        System.out.println("------- 仿真平台令牌信息为：" + simulationToken);
        SimulationTokenVO simulationTokenVO = JsonUtil.jsonToBean(simulationToken, SimulationTokenVO.class);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, simulationTokenVO);
    }

    /**
     * 服务监控
     */
    @RequestMapping ("online")
    @ResponseBody
    public ResponseBodyVO<Integer> online() {
        int online;

        Set<String> keys = redisTemplate.keys("access:*");
        if (CollectionUtil.isEmpty(keys)) {
            online = 0;
        } else {
            online = keys.size();
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, online);
    }

}
