package com.css.simulation.resource.scheduler.mapper;


import com.css.simulation.resource.scheduler.pojo.po.ProjectPO;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.type.JdbcType;

import java.sql.Timestamp;

@Mapper
public interface ProjectMapper {


    @Results(id = "project", value = {
            @Result(column = "id", property = "id", jdbcType = JdbcType.VARCHAR),
            @Result(column = "scene", property = "scenePackageId", jdbcType = JdbcType.VARCHAR)
    })

    @Select("select id, scene\n" +
            "from simulation_manual_project\n" +
            "where id = (select p_id from simulation_manual_project_task where id = #{taskId})\n")
    ProjectPO selectById(@Param("taskId")String taskId);

    @Update("update simulation_manual_project\n" +
            "set now_run_state  = #{state},\n" +
            "    task_completed = '0'\n" +
            "where id = #{id}")
    void resetProjectState(@Param("id") String id, @Param("state") String state);

    @Update("update simulation_manual_project\n" +
            "set now_run_state  = #{state},\n" +
            "    finish_time = #{finishTime}\n" +
            "where id = #{id}")
    void updateProjectState(@Param("id") String id, @Param("state") String state,@Param("finishTime") Timestamp finishTime);


    @Update("update simulation_manual_project\n" +
            "set task_number = #{taskNumber}, task_completed = 0\n" +
            "where id = #{id}")
    void updateTaskNumber(@Param("id") String id, @Param("taskNumber") int taskNumber);

    @Update("update simulation_manual_project\n" +
            "set task_completed = #{taskCompleted}\n" +
            "where id = #{id}")
    void updateTaskCompleted(@Param("id") String id, @Param("taskCompleted") int taskCompleted);


    @Select("select task_number\n" +
            "from simulation_manual_project\n" +
            "where id = #{id}")
    int selectTaskNumById(@Param("id") String id);

    @Select("select count(1)\n" +
            "from simulation_manual_project_task\n" +
            "where run_state in ('Aborted', 'PendingAnalysis', 'Terminated')\n" +
            "  and p_id = #{projectId}")
    int selectEndTaskNum(@Param("projectId") String projectId);
}
